"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@salesforce/command");
const core_1 = require("@salesforce/core");
const packageBuilder_1 = require("../../../../impl/metadata/packageBuilder");
// Initialize Messages with the current plugin directory
core_1.Messages.importMessagesDirectory(__dirname);
// Load the specific messages for this file. Messages from @salesforce/command, @salesforce/core,
// or any library that is using the messages framework can also be loaded this way.
const messages = core_1.Messages.loadMessages("sfpowerkit", "package_build");
class Build extends command_1.SfdxCommand {
    async run() {
        // this.org is guaranteed because requiresUsername=true, as opposed to supportsUsername
        const apiversion = await this.org.getConnection().retrieveMaxApiVersion();
        const conn = this.org.getConnection();
        const configs = new packageBuilder_1.BuildConfig(this.flags, apiversion);
        const packageXML = new packageBuilder_1.Packagexml(conn, configs);
        const result = await packageXML.build();
        //console.log(result);
        this.ux.log(result.toString());
        return { result: result.toString() };
    }
}
Build.description = messages.getMessage("commandDescription");
Build.examples = [
    `$ sfdx sfpowerkit:org:manifest:build --targetusername myOrg@example.com -o package.xml
    <?xml version="1.0" encoding="UTF-8"?>
    <Package xmlns="http://soap.sforce.com/2006/04/metadata">...</Package>
    `,
    `$ sfdx sfpowerkit:org:manifest:build --targetusername myOrg@example.com -o package.xml -q 'ApexClass, CustomObject, Report' 
    <?xml version="1.0" encoding="UTF-8"?>
    <Package xmlns="http://soap.sforce.com/2006/04/metadata">...</Package>
    `
];
Build.args = [{ name: "file" }];
Build.flagsConfig = {
    quickfilter: command_1.flags.string({
        char: "q",
        description: messages.getMessage("quickfilterFlagDescription")
    }),
    excludemanaged: command_1.flags.boolean({
        char: "x",
        description: messages.getMessage("excludeManagedFlagDescription")
    }),
    outputfile: command_1.flags.filepath({
        char: "o",
        description: messages.getMessage("outputFileFlagDescription")
    })
};
// Comment this out if your command does not require an org username
Build.requiresUsername = true;
// Comment this out if your command does not support a hub org username
Build.supportsDevhubUsername = false;
// Set this to true if your command requires a project workspace; 'requiresProject' is false by default
Build.requiresProject = false;
exports.default = Build;
//# sourceMappingURL=build.js.map