"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@salesforce/command");
const lodash_1 = __importDefault(require("lodash"));
const sfpowerkit_1 = require("../../../../sfpowerkit");
const path = __importStar(require("path"));
const metadataInfo_1 = require("../../../../impl/metadata/metadataInfo");
const profileSync_1 = __importDefault(require("../../../../impl/source/profiles/profileSync"));
// Initialize Messages with the current plugin directory
command_1.core.Messages.importMessagesDirectory(__dirname);
// Load the specific messages for this file. Messages from @salesforce/command, @salesforce/core,
// or any library that is using the messages framework can also be loaded this way.
const messages = command_1.core.Messages.loadMessages("sfpowerkit", "profile_retrieve");
class Retrieve extends command_1.SfdxCommand {
    async run() {
        sfpowerkit_1.SFPowerkit.ux = this.ux;
        let argFolder = this.flags.folder;
        let argProfileList = this.flags.profilelist;
        let folders = [];
        if (!lodash_1.default.isNil(argFolder) && argFolder.length !== 0) {
            sfpowerkit_1.SFPowerkit.setDefaultFolder(argFolder[0]);
            folders.push(...argFolder);
        }
        const profileUtils = new profileSync_1.default(this.org, this.flags.loglevel == "debug");
        let syncPofles = await profileUtils.sync(folders, argProfileList || [], this.flags.delete);
        let result = [];
        if (syncPofles.added) {
            syncPofles.added.forEach(file => {
                result.push({
                    state: "Add",
                    fullName: path.basename(file, metadataInfo_1.METADATA_INFO.Profile.sourceExtension),
                    type: "Profile",
                    path: path.relative(process.cwd(), file)
                });
            });
        }
        if (syncPofles.updated) {
            syncPofles.updated.forEach(file => {
                result.push({
                    state: "Updated",
                    fullName: path.basename(file, metadataInfo_1.METADATA_INFO.Profile.sourceExtension),
                    type: "Profile",
                    path: path.relative(process.cwd(), file)
                });
            });
        }
        if (syncPofles.deleted && this.flags.delete) {
            syncPofles.deleted.forEach(file => {
                result.push({
                    state: "Deleted",
                    fullName: path.basename(file, metadataInfo_1.METADATA_INFO.Profile.sourceExtension),
                    type: "Profile",
                    path: path.relative(process.cwd(), file)
                });
            });
        }
        return result;
    }
}
Retrieve.description = messages.getMessage("commandDescription");
Retrieve.examples = [
    `$ sfdx sfpowerkit:source:profile:retrieve -u prod`,
    `$ sfdx sfpowerkit:source:profile:retrieve  -f force-app -n "My Profile" -u prod`,
    `$ sfdx sfpowerkit:source:profile:retrieve  -f "module1, module2, module3" -n "My Profile1, My profile2"  -u prod`
];
//public static args = [{ name: 'file' }];
Retrieve.flagsConfig = {
    folder: command_1.flags.array({
        char: "f",
        description: messages.getMessage("folderFlagDescription"),
        required: false,
        map: (f) => f.trim()
    }),
    profilelist: command_1.flags.array({
        char: "n",
        description: messages.getMessage("profileListFlagDescription"),
        required: false,
        map: (p) => p.trim()
    }),
    delete: command_1.flags.boolean({
        char: "d",
        description: messages.getMessage("deleteFlagDescription"),
        required: false
    })
};
// Comment this out if your command does not require an org username
Retrieve.requiresUsername = true;
// Set this to true if your command requires a project workspace; 'requiresProject' is false by default
Retrieve.requiresProject = true;
Retrieve.result = {
    tableColumnData: {
        columns: [
            { key: "state", label: "State" },
            { key: "fullName", label: "Full Name" },
            { key: "type", label: "Type" },
            { key: "path", label: "Path" }
        ]
    },
    display() {
        if (Array.isArray(this.data) && this.data.length) {
            this.ux.table(this.data, this.tableColumnData);
        }
    }
};
exports.default = Retrieve;
//# sourceMappingURL=retrieve.js.map