"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = __importStar(require("fs"));
const xml2js = __importStar(require("xml2js"));
const util = require("util");
const diffUtil_1 = __importDefault(require("./diffUtil"));
var _ = require("lodash");
const parser = new xml2js.Parser({
    explicitArray: false,
    valueProcessors: [
        function (name) {
            if (name === "true")
                name = true;
            if (name === "false")
                name = false;
            return name;
        }
    ]
});
class WorkflowDiff {
    static async generateWorkflowXml(workflowXml1, workflowXml2, outputFilePath, objectName, destructivePackageObj, resultOutput) {
        let workflowObj1 = {};
        let workflowObj2 = {};
        const parseString = util.promisify(parser.parseString);
        if (workflowXml1 !== "") {
            let parseResult = await parseString(workflowXml1);
            workflowObj1 = parseResult.Workflow || {};
        }
        if (workflowXml2 !== "") {
            let parseResult = await parseString(workflowXml2);
            workflowObj2 = parseResult.Workflow || {};
        }
        let addedEditedOrDeleted = WorkflowDiff.buildNewWorkflowObj(workflowObj1, workflowObj2);
        WorkflowDiff.writeWorkflow(addedEditedOrDeleted.addedEdited, outputFilePath);
        destructivePackageObj = WorkflowDiff.buildDestructiveChangesObj(addedEditedOrDeleted.deleted, destructivePackageObj, objectName);
        WorkflowDiff.updateOutput(addedEditedOrDeleted.addedEdited, resultOutput, objectName, "Deploy", outputFilePath);
        WorkflowDiff.updateOutput(addedEditedOrDeleted.deleted, resultOutput, objectName, "Delete", "destructiveChanges.xml");
        return destructivePackageObj;
    }
    static updateOutput(workflowObj, resultOutput, objectName, action, filePath) {
        workflowObj.alerts.forEach(elem => {
            resultOutput.push({
                action: action,
                metadataType: "WorkflowAlert",
                componentName: `${objectName}.${elem.fullName}`,
                path: filePath
            });
        });
        workflowObj.fieldUpdates.forEach(elem => {
            resultOutput.push({
                action: action,
                metadataType: "WorkflowFieldUpdate",
                componentName: `${objectName}.${elem.fullName}`,
                path: filePath
            });
        });
        workflowObj.knowledgePublishes.forEach(elem => {
            resultOutput.push({
                action: action,
                metadataType: "WorkflowKnowledgePublish",
                componentName: `${objectName}.${elem.label}`,
                path: filePath
            });
        });
        workflowObj.outboundMessages.forEach(elem => {
            resultOutput.push({
                action: action,
                metadataType: "WorkflowOutboundMessage",
                componentName: `${objectName}.${elem.fullName}`,
                path: filePath
            });
        });
        workflowObj.rules.forEach(elem => {
            resultOutput.push({
                action: action,
                metadataType: "WorkflowRule",
                componentName: `${objectName}.${elem.fullName}`,
                path: filePath
            });
        });
        workflowObj.tasks.forEach(elem => {
            resultOutput.push({
                action: action,
                metadataType: "WorkflowTask",
                componentName: `${objectName}.${elem.fullName}`,
                path: filePath
            });
        });
    }
    static ensureArray(workflowObj) {
        let keys = Object.keys(workflowObj);
        keys.forEach(key => {
            if (typeof workflowObj[key] === "object" &&
                !Array.isArray(workflowObj[key]) &&
                key !== "$") {
                workflowObj[key] = [workflowObj[key]];
            }
        });
        return workflowObj;
    }
    static buildNewWorkflowObj(workflowObj1, workflowObj2) {
        workflowObj1 = WorkflowDiff.ensureArray(workflowObj1);
        workflowObj2 = WorkflowDiff.ensureArray(workflowObj2);
        let newWorkflowObj = {
            $: { xmlns: "http://soap.sforce.com/2006/04/metadata" },
            alerts: [],
            fieldUpdates: [],
            knowledgePublishes: [],
            outboundMessages: [],
            rules: [],
            tasks: []
        };
        let deletedWorkflowObj = {
            $: { xmlns: "http://soap.sforce.com/2006/04/metadata" },
            alerts: [],
            fieldUpdates: [],
            knowledgePublishes: [],
            outboundMessages: [],
            rules: [],
            tasks: []
        };
        if (workflowObj1.fullName !== undefined ||
            workflowObj2.fullName !== undefined) {
            newWorkflowObj.fullName = workflowObj2.fullName;
        }
        //Email alerts
        let addedDeleted = diffUtil_1.default.getChangedOrAdded(workflowObj1.alerts, workflowObj2.alerts, "fullName");
        newWorkflowObj.alerts = addedDeleted.addedEdited;
        deletedWorkflowObj.alerts = addedDeleted.deleted;
        //Field Update
        addedDeleted = diffUtil_1.default.getChangedOrAdded(workflowObj1.fieldUpdates, workflowObj2.fieldUpdates, "fullName");
        newWorkflowObj.fieldUpdates = addedDeleted.addedEdited;
        deletedWorkflowObj.fieldUpdates = addedDeleted.deleted;
        //Knowledge Publishes
        addedDeleted = diffUtil_1.default.getChangedOrAdded(workflowObj1.knowledgePublishes, workflowObj2.knowledgePublishes, "label");
        newWorkflowObj.knowledgePublishes = addedDeleted.addedEdited;
        deletedWorkflowObj.knowledgePublishes = addedDeleted.deleted;
        //Outbound Messages
        addedDeleted = diffUtil_1.default.getChangedOrAdded(workflowObj1.outboundMessages, workflowObj2.outboundMessages, "fullName");
        newWorkflowObj.outboundMessages = addedDeleted.addedEdited;
        deletedWorkflowObj.outboundMessages = addedDeleted.deleted;
        //Rules
        addedDeleted = diffUtil_1.default.getChangedOrAdded(workflowObj1.rules, workflowObj2.rules, "fullName");
        newWorkflowObj.rules = addedDeleted.addedEdited;
        deletedWorkflowObj.rules = addedDeleted.deleted;
        //Task
        addedDeleted = diffUtil_1.default.getChangedOrAdded(workflowObj1.tasks, workflowObj2.tasks, "fullName");
        newWorkflowObj.tasks = addedDeleted.addedEdited;
        deletedWorkflowObj.tasks = addedDeleted.deleted;
        return {
            addedEdited: newWorkflowObj,
            deleted: deletedWorkflowObj
        };
    }
    static buildDestructiveChangesObj(deletedWorkflows, destructivePackageObj, objectName) {
        destructivePackageObj = WorkflowDiff.buildDestructiveType(destructivePackageObj, deletedWorkflows.alerts, "WorkflowAlert", objectName);
        destructivePackageObj = WorkflowDiff.buildDestructiveType(destructivePackageObj, deletedWorkflows.fieldUpdates, "WorkflowFieldUpdate", objectName);
        destructivePackageObj = WorkflowDiff.buildDestructiveType(destructivePackageObj, deletedWorkflows.knowledgePublishes, "WorkflowKnowledgePublish", objectName);
        destructivePackageObj = WorkflowDiff.buildDestructiveType(destructivePackageObj, deletedWorkflows.outboundMessages, "WorkflowOutboundMessage", objectName);
        destructivePackageObj = WorkflowDiff.buildDestructiveType(destructivePackageObj, deletedWorkflows.rules, "WorkflowRule", objectName);
        destructivePackageObj = WorkflowDiff.buildDestructiveType(destructivePackageObj, deletedWorkflows.tasks, "WorkflowTask", objectName);
        return destructivePackageObj;
    }
    static buildDestructiveType(destructivePackageObj, list, typeLabel, objectName) {
        let metaType = _.find(destructivePackageObj, function (metaType) {
            return metaType.name === typeLabel;
        });
        if (metaType === undefined && list !== undefined && list.length > 0) {
            metaType = {
                name: typeLabel,
                members: []
            };
            destructivePackageObj.push(metaType);
        }
        if (list !== undefined) {
            list.forEach(elem => {
                metaType.members.push(objectName + "." + elem.fullName);
            });
        }
        return destructivePackageObj;
    }
    static writeWorkflow(newWorkflowObj, outputFilePath) {
        const builder = new xml2js.Builder({
            xmldec: { version: "1.0", encoding: "UTF-8", standalone: null }
        });
        let workflowObj = {
            Workflow: newWorkflowObj
        };
        let xml = builder.buildObject(workflowObj);
        fs.writeFileSync(outputFilePath, xml);
    }
}
exports.default = WorkflowDiff;
//# sourceMappingURL=workflowDiff.js.map